use anyhow::Result;
use std::io::Write;
use std::fs::File;
use crate::parser::{IdlInterface, IdlMethod, IdlParameter, ParameterDirection};
use crate::codegen::map_idl_type_to_rust;

/// 生成接口代码
pub fn generate_interface_code(file: &mut File, interface: &IdlInterface, indent: &str) -> Result<()> {
    // 在Rust中，接口对应的是trait
    writeln!(file, "{}/// 由IDL接口`{}`生成的trait", indent, interface.name)?;
    writeln!(file, "{}pub trait {} {{", indent, interface.name)?;
    
    // 生成接口中的方法
    let method_indent = format!("{}    ", indent);
    for method in &interface.methods {
        generate_method_code(file, method, &method_indent)?;
    }
    
    writeln!(file, "{}}}", indent)?;
    
    Ok(())
}

/// 生成方法代码
fn generate_method_code(file: &mut File, method: &IdlMethod, indent: &str) -> Result<()> {
    // 生成方法文档注释
    writeln!(file, "{}/// {}", indent, method.name)?;
    
    // 生成方法签名
    write!(file, "{}fn {}(", indent, method.name)?;
    
    // 生成方法参数
    write!(file, "&self")?;
    for param in &method.parameters {
        write!(file, ", ")?;
        // 先检查参数名称是否为空，如果为空，则使用一个默认的参数名称
        if param.name.is_empty() {
            // 使用一个默认的参数名称
            let param_name = "param";
            let mut param_with_name = param.clone();
            param_with_name.name = param_name.to_string();
            generate_parameter_code(file, &param_with_name)?;
        } else {
            generate_parameter_code(file, param)?;
        }
    }
    
    // 生成返回类型
    if method.return_type != "void" {
        let rust_return_type = map_idl_type_to_rust(&method.return_type);
        writeln!(file, ") -> {};", rust_return_type)?;
    } else {
        writeln!(file, ");", )?;
    }
    
    Ok(())
}

/// 生成参数代码
fn generate_parameter_code(file: &mut File, param: &IdlParameter) -> Result<()> {
    // 先处理参数类型，去除可能的方向修饰符
    let type_spec = param.type_spec.trim();
    let type_spec = if type_spec.starts_with("in ") {
        type_spec.trim_start_matches("in ").trim()
    } else if type_spec.starts_with("out ") {
        type_spec.trim_start_matches("out ").trim()
    } else if type_spec.starts_with("inout ") {
        type_spec.trim_start_matches("inout ").trim()
    } else {
        type_spec
    };
    
    // 根据参数方向生成参数修饰符
    match param.direction {
        ParameterDirection::In => {
            // 输入参数，使用不可变引用
            // 如果是字符串类型，使用&str而不是&String
            if type_spec.trim() == "string" {
                write!(file, "{}: &str", param.name)?;
            } else {
                let rust_type = map_idl_type_to_rust(type_spec);
                write!(file, "{}: &{}", param.name, rust_type)?;
            }
        },
        ParameterDirection::Out => {
            // 输出参数，使用可变引用
            // 如果是字符串类型，使用&mut String
            if type_spec.trim() == "string" {
                write!(file, "{}: &mut String", param.name)?;
            } else {
                let rust_type = map_idl_type_to_rust(type_spec);
                write!(file, "{}: &mut {}", param.name, rust_type)?;
            }
        },
        ParameterDirection::InOut => {
            // 输入输出参数，使用可变引用
            // 如果是字符串类型，使用&mut String
            if type_spec.trim() == "string" {
                write!(file, "{}: &mut String", param.name)?;
            } else {
                let rust_type = map_idl_type_to_rust(type_spec);
                write!(file, "{}: &mut {}", param.name, rust_type)?;
            }
        }
    }
    
    Ok(())
}
