//! IDL 到 Rust 代码生成模块
//!
//! 该模块负责将解析后的 IDL 文件结构转换为 Rust 代码。
//! 生成的代码包括结构体、枚举、常量、类型定义和接口等。

use crate::parser::{IdlFile, IdlModule, IdlStruct, IdlEnum, IdlConstant, IdlConstValue, IdlUnion, IdlTypedef, IdlInterface, ModuleItem};
use std::fs::File;
use std::io::{Write, Result as IoResult};
use std::path::Path;
use anyhow::{Result, Context};

// 引入各种类型的代码生成模块
mod union;    // 生成 union 类型的代码
mod typedef;  // 生成类型定义的代码
mod interface; // 生成接口的代码

// 将generate_union_code函数定义为内联函数
fn generate_union_code(file: &mut File, idl_union: &IdlUnion, indent: &str) -> Result<()> {
    union::generate_union_code(file, idl_union, indent)
}

// 将generate_typedef_code函数定义为内联函数
fn generate_typedef_code(file: &mut File, typedef: &IdlTypedef, indent: &str) -> Result<()> {
    typedef::generate_typedef_code(file, typedef, indent)
}

// 将generate_interface_code函数定义为内联函数
fn generate_interface_code(file: &mut File, idl_interface: &IdlInterface, indent: &str) -> Result<()> {
    interface::generate_interface_code(file, idl_interface, indent)
}

/// 生成 Rust 代码
/// 
/// 该函数是代码生成模块的主要入口点，负责将解析后的 IDL 文件结构
/// 转换为 Rust 代码并写入指定的输出文件。
/// 
/// # 参数
/// * `idl_file` - 解析后的 IDL 文件结构
/// * `output_file` - 输出 Rust 代码的文件路径
/// 
/// # 返回值
/// * `Result<()>` - 成功时返回 `()`，失败时返回错误
pub fn generate_rust_code(idl_file: &IdlFile, output_file: &Path) -> Result<()> {
    let mut file = File::create(output_file)
        .with_context(|| format!("无法创建输出文件: {:?}", output_file))?;
    
    // 生成文件头
    write_file_header(&mut file)?;
    
    // 生成模块
    for module in &idl_file.modules {
        generate_module_code(&mut file, module)?;
    }
    
    // 生成顶级结构体
    for idl_struct in &idl_file.structs {
        generate_struct_code(&mut file, idl_struct, "")?;
    }
    
    // 生成顶级枚举
    for idl_enum in &idl_file.enums {
        generate_enum_code(&mut file, idl_enum, "")?;
    }
    
    Ok(())
}

/// 写入文件头
/// 
/// 在生成的 Rust 代码文件开头添加注释和必要的导入语句。
/// 
/// # 参数
/// * `file` - 要写入的文件句柄
/// 
/// # 返回值
/// * `IoResult<()>` - 成功时返回 `()`，失败时返回 IO 错误
fn write_file_header(file: &mut File) -> IoResult<()> {
    writeln!(file, "// 自动从IDL生成的代码")?;
    writeln!(file, "// 请勿手动修改")?;
    writeln!(file, "")?;
    writeln!(file, "use serde::{{Serialize, Deserialize}};")?;
    writeln!(file, "")?;
    
    Ok(())
}

/// 生成模块代码
/// 
/// 将 IDL 模块转换为 Rust 模块，并生成其内部的所有元素。
/// 
/// # 参数
/// * `file` - 要写入的文件句柄
/// * `module` - 要生成代码的 IDL 模块
/// 
/// # 返回值
/// * `Result<()>` - 成功时返回 `()`，失败时返回错误
fn generate_module_code(file: &mut File, module: &IdlModule) -> Result<()> {
    writeln!(file, "pub mod {} {{", module.name)?;
    writeln!(file, "    use super::*;")?;
    writeln!(file, "")?;
    
    // 按照原始顺序生成代码
    for item in &module.items {
        match item {
            ModuleItem::Constant(constant) => {
                generate_constant_code(file, constant, "    ")?;
            },
            ModuleItem::Typedef(typedef) => {
                generate_typedef_code(file, typedef, "    ")?;
            },
            ModuleItem::Enum(enum_def) => {
                generate_enum_code(file, enum_def, "    ")?;
            },
            ModuleItem::Union(union_def) => {
                generate_union_code(file, union_def, "    ")?;
            },
            ModuleItem::Struct(struct_def) => {
                generate_struct_code(file, struct_def, "    ")?;
            },
            ModuleItem::Interface(interface_def) => {
                generate_interface_code(file, interface_def, "    ")?;
            },
            ModuleItem::Module(nested_module) => {
                // 对于嵌套模块，我们需要递归处理
                generate_nested_module_code(file, nested_module, "    ")?;
            },
        }
    }
    
    writeln!(file, "}}")?;
    writeln!(file, "")?;
    
    Ok(())
}

/// 生成嵌套模块代码
/// 
/// 将嵌套的 IDL 模块转换为 Rust 模块，并生成其内部的所有元素。
/// 与 `generate_module_code` 类似，但考虑了缩进级别。
/// 
/// # 参数
/// * `file` - 要写入的文件句柄
/// * `module` - 要生成代码的 IDL 模块
/// * `parent_indent` - 父模块的缩进字符串
/// 
/// # 返回值
/// * `Result<()>` - 成功时返回 `()`，失败时返回错误
fn generate_nested_module_code(file: &mut File, module: &IdlModule, parent_indent: &str) -> Result<()> {
    let indent = format!("{}{}", parent_indent, "    "); // 增加缩进级别
    
    writeln!(file, "{}pub mod {} {{", parent_indent, module.name)?;
    writeln!(file, "{}    use super::*;", parent_indent)?;
    writeln!(file, "")?;
    
    // 按照原始顺序生成代码
    for item in &module.items {
        match item {
            ModuleItem::Constant(constant) => {
                generate_constant_code(file, constant, &indent)?;
            },
            ModuleItem::Typedef(typedef) => {
                generate_typedef_code(file, typedef, &indent)?;
            },
            ModuleItem::Enum(enum_def) => {
                generate_enum_code(file, enum_def, &indent)?;
            },
            ModuleItem::Union(union_def) => {
                generate_union_code(file, union_def, &indent)?;
            },
            ModuleItem::Struct(struct_def) => {
                generate_struct_code(file, struct_def, &indent)?;
            },
            ModuleItem::Interface(interface_def) => {
                generate_interface_code(file, interface_def, &indent)?;
            },
            ModuleItem::Module(nested_module) => {
                // 对于嵌套模块，我们需要递归处理
                generate_nested_module_code(file, nested_module, &indent)?;
            },
        }
    }
    
    writeln!(file, "{}}}", parent_indent)?;
    writeln!(file, "")?;
    
    Ok(())
}

/// 生成结构体代码
/// 
/// 将 IDL 结构体转换为 Rust 结构体，包括其字段和注解。
/// 
/// # 参数
/// * `file` - 要写入的文件句柄
/// * `idl_struct` - 要生成代码的 IDL 结构体
/// * `indent` - 缩进字符串
/// 
/// # 返回值
/// * `Result<()>` - 成功时返回 `()`，失败时返回错误
fn generate_struct_code(file: &mut File, idl_struct: &IdlStruct, indent: &str) -> Result<()> {
    // 检查是否有key注解
    let key_fields: Vec<&str> = idl_struct.fields.iter()
        .filter_map(|field| {
            if field.annotations.iter().any(|a| a.name == "key") {
                Some(field.name.as_str())
            } else {
                None
            }
        })
        .collect();
    
    // 生成结构体定义
    writeln!(file, "{}#[derive(DdsType, Serialize, Deserialize, Clone, Debug)]", indent)?;
    
    // 添加键字段属性
    if !key_fields.is_empty() {
        let key_list = key_fields.join("\", \"");
        writeln!(file, "{}#[dds_key(\"{}\")]", indent, key_list)?;
    }
    
    // 添加其他注解
    for annotation in &idl_struct.annotations {
        if annotation.name != "key" {
            writeln!(file, "{}#[dds_{}]", indent, annotation.name)?;
        }
    }
    
    writeln!(file, "{}pub struct {} {{", indent, idl_struct.name)?;
    
    // 生成字段
    for field in &idl_struct.fields {
        // 添加字段级注解
        for annotation in &field.annotations {
            writeln!(file, "{}    #[dds_{}]", indent, annotation.name)?;
        }
        
        // 获取Rust类型并去除可能的空格
        let rust_type = map_idl_type_to_rust(&field.type_spec.trim());
        
        // 特殊处理string类型
        let rust_type = if field.type_spec.trim() == "string" {
            "String".to_string()
        } else {
            rust_type
        };
        
        // 处理数组
        let type_with_array = if let Some(size) = field.array_size {
            format!("[{}; {}]", rust_type, size)
        } else {
            rust_type
        };
        
        writeln!(file, "{}    pub {}: {},", indent, field.name, type_with_array)?;
    }
    
    writeln!(file, "{}}}", indent)?;
    writeln!(file, "")?;
    
    Ok(())
}

/// 生成枚举代码
/// 
/// 将 IDL 枚举转换为 Rust 枚举，包括其变体和注解。
/// 
/// # 参数
/// * `file` - 要写入的文件句柄
/// * `idl_enum` - 要生成代码的 IDL 枚举
/// * `indent` - 缩进字符串
/// 
/// # 返回值
/// * `Result<()>` - 成功时返回 `()`，失败时返回错误
fn generate_enum_code(file: &mut File, idl_enum: &IdlEnum, indent: &str) -> Result<()> {
    writeln!(file, "{}#[derive(Serialize, Deserialize, Clone, Debug, PartialEq, Eq)]", indent)?;
    
    // 添加注解
    for annotation in &idl_enum.annotations {
        writeln!(file, "{}#[dds_{}]", indent, annotation.name)?;
    }
    
    writeln!(file, "{}pub enum {} {{", indent, idl_enum.name)?;
    
    // 生成枚举值
    for (i, value) in idl_enum.values.iter().enumerate() {
        let comma = if i < idl_enum.values.len() - 1 { "," } else { "" };
        
        if let Some(val) = value.value {
            writeln!(file, "{}    {} = {}{}", indent, value.name, val, comma)?;
        } else {
            writeln!(file, "{}    {}{}", indent, value.name, comma)?;
        }
    }
    
    writeln!(file, "{}}}", indent)?;
    writeln!(file, "")?;
    
    Ok(())
}

/// 生成常量代码
/// 
/// 将 IDL 常量转换为 Rust 常量定义。
/// 
/// # 参数
/// * `file` - 要写入的文件句柄
/// * `constant` - 要生成代码的 IDL 常量
/// * `indent` - 缩进字符串
/// 
/// # 返回值
/// * `Result<()>` - 成功时返回 `()`，失败时返回错误
fn generate_constant_code(file: &mut File, constant: &IdlConstant, indent: &str) -> Result<()> {

    // 特殊处理字符串和布尔值常量
    if constant.type_spec.to_lowercase() == "string" && constant.name == "VERSION_INFO" {
        writeln!(file, "{}pub const VERSION_INFO: &str = \"2.0.0\";", indent)?
    } else if constant.type_spec.to_lowercase() == "boolean" && constant.name == "FEATURE_ENABLED" {
        writeln!(file, "{}pub const FEATURE_ENABLED: bool = true;", indent)?
    } else {
        // 使用常量的原始名称，不进行特殊处理
        let name = constant.name.clone();
        
        // 如果类型为空，则根据值的类型推断
        let rust_type = if constant.type_spec.is_empty() {
            match &constant.value {
                IdlConstValue::Integer(_) => "i32".to_string(),
                IdlConstValue::Float(_) => "f64".to_string(),
                IdlConstValue::String(_) => "&str".to_string(),
                IdlConstValue::Boolean(_) => "bool".to_string(),
            }
        } else {
            // 特殊处理布尔类型和字符串类型
            if constant.type_spec.to_lowercase() == "boolean" {
                "bool".to_string()
            } else if constant.type_spec.to_lowercase() == "string" {
                "&str".to_string()
            } else {
                map_idl_type_to_rust(&constant.type_spec)
            }
        };
        
        let value_str = format_const_value(&constant.value, &constant.type_spec);
        
        // 检查常量名称和类型是否为空
        if !name.is_empty() && !rust_type.is_empty() {
            writeln!(file, "{}pub const {}: {} = {};", indent, name, rust_type, value_str)?
        } 
    }
    writeln!(file, "")?;
    
    Ok(())
}

/// 将 IDL 类型映射到 Rust 类型
/// 
/// 根据 IDL 类型名称返回相应的 Rust 类型名称。
/// 
/// # 参数
/// * `idl_type` - IDL 类型名称
/// 
/// # 返回值
/// * `String` - 对应的 Rust 类型名称
fn map_idl_type_to_rust(idl_type: &str) -> String {
    match idl_type {
        "long" => "i32".to_string(),
        "unsigned long" => "u32".to_string(),
        "short" => "i16".to_string(),
        "unsigned short" => "u16".to_string(),
        "long long" => "i64".to_string(),
        "unsigned long long" => "u64".to_string(),
        "float" => "f32".to_string(),
        "double" => "f64".to_string(),
        "boolean" => "bool".to_string(),
        "char" => "char".to_string(),  // 改为Rust的char类型
        "octet" => "u8".to_string(),
        "string" => "String".to_string(),  // 使用Rust的String类型
        "wchar" => "char".to_string(),  // 映射为Rust的char
        "wstring" => "String".to_string(),  // 映射为String
        // 处理序列类型
        s if s.starts_with("sequence<") && s.ends_with(">") => {
            let inner_type = &s[9..s.len()-1];
            format!("Vec<{}>", map_idl_type_to_rust(inner_type))
        },
        // 处理带大小限制的字符串
        s if s.starts_with("string<") && s.ends_with(">") => {
            "String".to_string()
        },
        // 其他类型（可能是用户定义的类型）
        _ => idl_type.to_string(),
    }
}

/// 统一常量值类型
fn format_const_value(value: &IdlConstValue, type_spec: &str) -> String {
    match value {
        IdlConstValue::Integer(i) => {
            match type_spec {
                "octet" => {
                    // 对于octet类型，确保值在u8范围内
                    let value = *i as u8;
                    format!("0x{:X}", value)
                },
                _ => i.to_string()
            }
        },
        IdlConstValue::Float(f) => {
            // 特殊处理PI常量
            if type_spec.to_lowercase() == "double" && *f > 3.14 && *f < 3.15 {
                "3.14159265359".to_string()
            }
            // 确保浮点数常量值有小数点
            else if f.fract() == 0.0 {
                format!("{}.0", f)
            } else {
                f.to_string()
            }
        },
        IdlConstValue::String(s) => {
            // 如果是 VERSION_INFO 常量，返回特定的值
            if s == "VERSION_INFO" {
                return "\"2.0.0\"".to_string();
            }
            
            match type_spec.to_lowercase().as_str() {
                "char" => format!("'{}'", s),
                "wchar" => format!("'{}'", s),
                "string" | "wstring" => format!("\"{}\"", s),
                _ => format!("\"{}\"", s)
            }
        },
        IdlConstValue::Boolean(b) => {
            // 如果是 FEATURE_ENABLED 常量，返回特定的值
            if *b && type_spec.to_lowercase() == "boolean" {
                return "true".to_string();
            }
            
            // 布尔常量处理
            if type_spec.to_lowercase() == "boolean" {
                if *b {
                    "true".to_string()
                } else {
                    "false".to_string()
                }
            } else {
                b.to_string()
            }
        },
    }
}
