use crate::parser::IdlUnion;
use std::fs::File;
use std::io::Write;
use anyhow::Result;

/// 生成联合类型代码
pub fn generate_union_code(file: &mut File, idl_union: &IdlUnion, indent: &str) -> Result<()> {
    // 生成联合类型的文档注释
    writeln!(file, "{}/// 由IDL联合类型`{}`生成的枚举", indent, idl_union.name)?;
    
    // 生成枚举定义开始
    writeln!(file, "{}#[derive(Serialize, Deserialize, Clone, Debug)]", indent)?;
    writeln!(file, "{}pub enum {} {{", indent, idl_union.name)?;
    
    // 生成枚举变体
    for case in &idl_union.cases {
        if case.is_default {
            // 默认分支
            writeln!(file, "{}    {}({}),", indent, "Default", map_idl_type_to_rust(&case.field.type_spec))?;
        } else {
            // 普通分支
            for label in &case.labels {
                writeln!(file, "{}    {}({}),", indent, format_variant_name(&case.field.name, label), map_idl_type_to_rust(&case.field.type_spec))?;
            }
        }
    }
    
    // 生成枚举定义结束
    writeln!(file, "{}}}", indent)?;
    writeln!(file, "")?;
    
    // 生成实现代码
    writeln!(file, "{}impl {} {{", indent, idl_union.name)?;
    
    // 生成构造函数
    for case in &idl_union.cases {
        if case.is_default {
            // 默认分支构造函数
            writeln!(file, "{}    /// 创建默认分支", indent)?;
            writeln!(file, "{}    pub fn new_default(value: {}) -> Self {{", indent, map_idl_type_to_rust(&case.field.type_spec))?;
            writeln!(file, "{}        {}::Default(value)", indent, idl_union.name)?;
            writeln!(file, "{}    }}", indent)?;
        } else {
            // 普通分支构造函数
            for label in &case.labels {
                let variant_name = format_variant_name(&case.field.name, label);
                writeln!(file, "{}    /// 创建分支 case {}", indent, label)?;
                writeln!(file, "{}    pub fn new_{}(value: {}) -> Self {{", indent, variant_name.to_lowercase(), map_idl_type_to_rust(&case.field.type_spec))?;
                writeln!(file, "{}        {}::{}(value)", indent, idl_union.name, variant_name)?;
                writeln!(file, "{}    }}", indent)?;
            }
        }
    }
    
    // 生成获取值的方法
    writeln!(file, "")?;
    writeln!(file, "{}    /// 获取判别器值", indent)?;
    writeln!(file, "{}    pub fn discriminator(&self) -> {} {{", indent, map_idl_type_to_rust(&idl_union.discriminator_type))?;
    writeln!(file, "{}        match self {{", indent)?;
    
    for case in &idl_union.cases {
        if case.is_default {
            // 默认分支
            writeln!(file, "{}            {}::Default(_) => 0,", indent, idl_union.name)?;
        } else {
            // 普通分支
            for label in &case.labels {
                writeln!(file, "{}            {}::{}(_) => {},", indent, idl_union.name, format_variant_name(&case.field.name, label), label)?;
            }
        }
    }
    
    writeln!(file, "{}        }}", indent)?;
    writeln!(file, "{}    }}", indent)?;
    
    // 生成实现代码结束
    writeln!(file, "{}}}", indent)?;
    writeln!(file, "")?;
    
    Ok(())
}

/// 格式化枚举变体名称
fn format_variant_name(field_name: &str, label: &str) -> String {
    // 首字母大写
    let mut chars = field_name.chars();
    let field_name = match chars.next() {
        None => String::new(),
        Some(f) => f.to_uppercase().collect::<String>() + chars.as_str(),
    };
    
    // 添加标签
    format!("Case{}{}", label, field_name)
}

/// 将IDL类型映射到Rust类型
fn map_idl_type_to_rust(idl_type: &str) -> String {
    match idl_type {
        "long" => "i32".to_string(),
        "unsigned long" => "u32".to_string(),
        "short" => "i16".to_string(),
        "unsigned short" => "u16".to_string(),
        "long long" => "i64".to_string(),
        "unsigned long long" => "u64".to_string(),
        "float" => "f32".to_string(),
        "double" => "f64".to_string(),
        "boolean" => "bool".to_string(),
        "char" => "char".to_string(),
        "octet" => "u8".to_string(),
        "string" => "String".to_string(),
        "wchar" => "char".to_string(),
        "wstring" => "String".to_string(),
        // 处理序列类型
        s if s.starts_with("sequence<") && s.ends_with(">") => {
            let inner_type = &s[9..s.len()-1];
            format!("Vec<{}>", map_idl_type_to_rust(inner_type))
        },
        // 处理带大小限制的字符串
        s if s.starts_with("string<") && s.ends_with(">") => {
            "String".to_string()
        },
        // 其他类型（可能是用户定义的类型）
        _ => idl_type.to_string(),
    }
}
