//! IDL Union 类型解析模块
//!
//! 该模块负责解析 IDL 文件中的 union 定义，并将其转换为内部数据结构。

use pest::iterators::Pair;
use crate::parser::{Rule, IdlUnion, IdlUnionCase, IdlField};

/// 处理 union 定义
/// 
/// 将 pest 解析器生成的 union 定义转换为内部的 IdlUnion 结构。
/// 
/// # 参数
/// * `pair` - pest 解析器生成的 union 定义节点
/// 
/// # 返回值
/// * `IdlUnion` - 解析后的 union 结构
pub fn process_union(pair: Pair<Rule>) -> IdlUnion {
    let mut union = IdlUnion {
        name: String::new(),
        discriminator_type: String::new(),
        cases: Vec::new(),
    };
        
    // 遍历union定义内部元素
    let mut inner_pairs = pair.into_inner();
    
    // 获取union名称
    if let Some(name_pair) = inner_pairs.next() {
        if name_pair.as_rule() == Rule::identifier {
            union.name = name_pair.as_str().to_string();
        }
    }
    
    // 获取判别器类型
    if let Some(disc_pair) = inner_pairs.next() {
        if disc_pair.as_rule() == Rule::type_spec {
            union.discriminator_type = disc_pair.as_str().to_string();
        }
    }
    
    // 遍历分支定义
    for case_pair in inner_pairs {
        if case_pair.as_rule() == Rule::union_case {
            let case = process_union_case(case_pair);
            union.cases.push(case);
        }
    }
    
    union
}

/// 处理 union 分支
/// 
/// 将 pest 解析器生成的 union case 定义转换为内部的 IdlUnionCase 结构。
/// union case 包含标签值和字段定义。
/// 
/// # 参数
/// * `pair` - pest 解析器生成的 union case 定义节点
/// 
/// # 返回值
/// * `IdlUnionCase` - 解析后的 union case 结构
pub fn process_union_case(pair: Pair<Rule>) -> IdlUnionCase {
    let mut case = IdlUnionCase {
        labels: Vec::new(),
        is_default: false,
        field: IdlField {
            name: String::new(),
            type_spec: String::new(),
            array_size: None,
            annotations: Vec::new(),
        },
    };
        
    // 检查是否是default分支
    let case_str = pair.as_str();
    if case_str.contains("default") {
        case.is_default = true;
        // default分支已设置
    }
    
    // 遍历分支定义内部元素
    for inner_pair in pair.into_inner() {
        // 处理case元素
        
        match inner_pair.as_rule() {
            Rule::const_value => {
                // 处理case标签值
                let label = inner_pair.as_str().trim();
                case.labels.push(label.to_string());
                // case标签值已提取
            },
            Rule::type_spec => {
                // 处理字段类型
                case.field.type_spec = inner_pair.as_str().trim().to_string();
                // 字段类型已设置
            },
            Rule::identifier => {
                // 处理字段名称
                case.field.name = inner_pair.as_str().trim().to_string();
                // 字段名称已设置
            },
            _ => {
                // 其他未知元素
                // 忽略未知元素
            }
        }
    }
    
    case
}
